lkml.org 
[lkml]   [2008]   [May]   [9]   [last100]   RSS-feed
Views: [more markup]  [less markup]  [headers]  [forward] 
 
Messages in this thread
Complete thread (EXPERIMENTAL)
Patch in this message
/
DateSat, 10 May 2008 01:13:57 +0300
FromEduard - Gabriel Munteanu <>
SubjectUsing markers w/ preemptible RCU in early code
Digg This
Hello,

I'm working on the kmemtrace GSoC project and I'm having a little problem.

My code inserts probes during early code (just after kmem_cache_init()). This works on the classic RCU. But on the preemptible RCU, this triggers BUG()s (supposedly on each probe, there are two), although the kernel runs fine:

BUG: scheduling while atomic: swapper/0/0x00000002
Pid: 0, comm: swapper Not tainted 2.6.25-00002-gf80e324-dirty #16
Call Trace:
[<ffffffff802365f5>] __schedule_bug+0x65/0x70 [<ffffffff804f8bb0>] thread_return+0x346/0x566 [<ffffffff802734ba>] ? get_marker+0x23a/0x260 [<ffffffff8025fbc6>] ? put_online_cpus+0x46/0x70 [<ffffffff80270c88>] __synchronize_sched+0x48/0x80 [<ffffffff802741a2>] marker_probe_register+0x152/0x660 [<ffffffff8029af10>] ? kmemtrace_probe_alloc+0x0/0x1b0 [<ffffffff8029ad6d>] kmemtrace_init+0x4d/0xd0 [<ffffffff80693c05>] start_kernel+0x205/0x300 [<ffffffff806931b2>] _sinittext+0x1b2/0x200

The is triggered by the following code in marker_probe_register(): #ifdef CONFIG_PREEMPT_RCU
	synchronize_sched();    /* Until we have the call_rcu_sched() */
#endif
Since preemption and SMP are disabled during early code, we could do something like:

[view this diff only]
diff --git a/kernel/marker.c b/kernel/marker.c
index 005b959..84964dc 100644
--- a/kernel/marker.c
+++ b/kernel/marker.c
@@ -23,6 +23,7 @@
 #include <linux/rcupdate.h>
 #include <linux/marker.h>
 #include <linux/err.h>
+#include <linux/hardirq.h>
 
 extern struct marker __start___markers[];
 extern struct marker __stop___markers[];
@@ -672,7 +673,9 @@ int marker_probe_register(const char *name, const char *format,
 	/* write rcu_pending before calling the RCU callback */
 	smp_wmb();
 #ifdef CONFIG_PREEMPT_RCU
-	synchronize_sched();	/* Until we have the call_rcu_sched() */
+	/* We are not preemptible when registering probes in early code */
+	if (likely(preemptible()))
+		synchronize_sched();	/* Until we have the call_rcu_sched() */
 #endif
 	call_rcu(&entry->rcu, free_old_closure);
 end:
Is this the best approach? marker_probe_register() isn't a hot path, so that check won't mess up the performance. Anyway, this works for me.

Eduard

(forgot LKML, added now.)



\
ISP Services
Valid XHTML 1.0!\ /
Valid CSS! Last update: 2008-05-10 00:19    [W:0.235 / U:0.020 seconds]
©2003-2005 Jasper Spaans